@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2007, Jari Sundell
+// Copyright (C) 2005-2011, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@
 #include "torrent/connection_manager.h"
 #include "torrent/download_info.h"
 #include "torrent/throttle.h"
+#include "torrent/download/choke_group.h"
 #include "torrent/download/choke_queue.h"
 #include "torrent/peer/peer_info.h"
 #include "torrent/peer/connection_list.h"
@@ -122,6 +123,9 @@ PeerConnectionBase::initialize(DownloadMain* download, PeerInfo* peerInfo, Socke
 
   m_extensions->set_connection(this);
 
+  m_upChoke.set_entry(m_download->up_group_entry());
+  m_downChoke.set_entry(m_download->down_group_entry());
+
   m_peerChunks.set_peer_info(m_peerInfo);
   m_peerChunks.bitfield()->swap(*bitfield);
 
@@ -184,8 +188,11 @@ PeerConnectionBase::cleanup() {
   up_chunk_release();
   down_chunk_release();
 
-  m_download->upload_choke_manager()->disconnected(this, &m_upChoke);
-  m_download->download_choke_manager()->disconnected(this, &m_downChoke);
+  m_download->info()->set_upload_unchoked(m_download->info()->upload_unchoked() - m_upChoke.unchoked());
+  m_download->info()->set_download_unchoked(m_download->info()->download_unchoked() - m_downChoke.unchoked());
+
+  m_download->choke_group()->up_queue()->disconnected(this, &m_upChoke);
+  m_download->choke_group()->down_queue()->disconnected(this, &m_downChoke);
   m_download->chunk_statistics()->received_disconnect(&m_peerChunks);
 
   if (!m_extensions->is_default())
@@ -213,9 +220,9 @@ PeerConnectionBase::cleanup() {
 void
 PeerConnectionBase::set_upload_snubbed(bool v) {
   if (v)
-    m_download->upload_choke_manager()->set_snubbed(this, &m_upChoke);
+    m_download->choke_group()->up_queue()->set_snubbed(this, &m_upChoke);
   else
-    m_download->upload_choke_manager()->set_not_snubbed(this, &m_upChoke);
+    m_download->choke_group()->up_queue()->set_not_snubbed(this, &m_upChoke);
 }
 
 bool
@@ -229,6 +236,23 @@ PeerConnectionBase::receive_upload_choke(bool choke) {
   m_upChoke.set_unchoked(!choke);
   m_upChoke.set_time_last_choke(cachedTime.usec());
 
+  if (choke) {
+    m_download->info()->set_upload_unchoked(m_download->info()->upload_unchoked() - 1);
+    m_upChoke.entry()->connection_choked(this);
+    m_upChoke.entry()->connection_queued(this);
+
+    m_download->choke_group()->up_queue()->modify_currently_unchoked(-1);
+    m_download->choke_group()->up_queue()->modify_currently_queued(1);
+
+  } else {
+    m_download->info()->set_upload_unchoked(m_download->info()->upload_unchoked() + 1);
+    m_upChoke.entry()->connection_unqueued(this);
+    m_upChoke.entry()->connection_unchoked(this);
+
+    m_download->choke_group()->up_queue()->modify_currently_unchoked(1);
+    m_download->choke_group()->up_queue()->modify_currently_queued(-1);
+  }
+
   return true;
 }
 
@@ -242,6 +266,23 @@ PeerConnectionBase::receive_download_choke(bool choke) {
   m_downChoke.set_unchoked(!choke);
   m_downChoke.set_time_last_choke(cachedTime.usec());
 
+  if (choke) {
+    m_download->info()->set_download_unchoked(m_download->info()->download_unchoked() - 1);
+    m_downChoke.entry()->connection_choked(this);
+    m_downChoke.entry()->connection_queued(this);
+
+    m_download->choke_group()->down_queue()->modify_currently_unchoked(-1);
+    m_download->choke_group()->down_queue()->modify_currently_queued(1);
+
+  } else {
+    m_download->info()->set_download_unchoked(m_download->info()->download_unchoked() + 1);
+    m_downChoke.entry()->connection_unqueued(this);
+    m_downChoke.entry()->connection_unchoked(this);
+
+    m_download->choke_group()->down_queue()->modify_currently_unchoked(1);
+    m_download->choke_group()->down_queue()->modify_currently_queued(-1);
+  }
+
   if (choke) {
     m_peerChunks.download_cache()->disable();
 
@@ -268,7 +309,9 @@ PeerConnectionBase::receive_download_choke(bool choke) {
       // Remove from queue so that an unchoke from the remote peer
       // will cause the connection to be unchoked immediately by the
       // choke manager.
-      m_downChoke.set_queued(false);
+      //
+      // TODO: This doesn't seem safe...
+      m_download->choke_group()->down_queue()->set_not_queued(this, &m_downChoke);
       return false;
     }
 
@@ -366,6 +409,17 @@ PeerConnectionBase::event_error() {
   m_download->connection_list()->erase(this, 0);
 }
 
+bool
+PeerConnectionBase::should_connection_unchoke(choke_queue* cq) const {
+  if (cq == m_download->choke_group()->up_queue())
+    return m_download->info()->upload_unchoked() < m_download->up_group_entry()->max_slots();
+
+  if (cq == m_download->choke_group()->down_queue())
+    return m_download->info()->download_unchoked() < m_download->down_group_entry()->max_slots();
+
+  return true;
+}
+
 bool
 PeerConnectionBase::down_chunk_start(const Piece& piece) {
   if (!download_queue()->downloading(piece)) {
@@ -562,7 +616,7 @@ PeerConnectionBase::down_chunk_skip_process(const void* buffer, uint32_t length)
   }
 
   if (!transfer->block()->is_transfering())
-    throw internal_error("PeerConnectionBase::down_chunk_skip_process(...) block is not transfering, yet we have non-leaders.");
+    throw internal_error("PeerConnectionBase::down_chunk_skip_process(...) block is not transferring, yet we have non-leaders.");
 
   // Temporary test.
   if (transfer->position() > transfer->block()->leader()->position())
