@@ -81,6 +81,7 @@ public:
   bool                is_in_use() const         { return is_enabled() && m_success_counter != 0; }
 
   virtual bool        is_busy() const = 0;
+  bool                is_busy_not_scrape() const { return m_latest_event != EVENT_SCRAPE && is_busy(); }
   virtual bool        is_usable() const { return is_enabled(); }
 
   bool                can_request_state() const;
@@ -135,6 +136,10 @@ protected:
   virtual void        send_state(int state) = 0;
   virtual void        send_scrape();
   virtual void        close() = 0;
+  virtual void        disown() = 0;
+
+  // Safeguard to catch bugs that lead to hammering of trackers.
+  void                inc_request_counter();
 
   void                clear_stats();
 
@@ -170,6 +175,11 @@ protected:
   uint32_t            m_scrape_complete;
   uint32_t            m_scrape_incomplete;
   uint32_t            m_scrape_downloaded;
+
+  // Timing of the last request, and a counter for how many requests
+  // there's been in the recent past.
+  uint32_t            m_request_time_last;
+  uint32_t            m_request_counter;
 };
 
 inline bool
