@@ -1,6 +1,7 @@
 #include "config.h"
 
 #include <iostream>
+#include <tr1/functional>
 #include <torrent/tracker_controller.h>
 
 #include "rak/priority_queue_default.h"
@@ -9,11 +10,11 @@
 #include "tracker_list_test.h"
 #include "tracker_controller_test.h"
 
-namespace std { using namespace tr1; }
+namespace tr1 { using namespace std::tr1; }
 
 CPPUNIT_TEST_SUITE_REGISTRATION(tracker_controller_test);
 
-#define TRACKER_SETUP()                                                 \
+#define TRACKER_CONTROLLER_SETUP()                                                 \
   torrent::TrackerList tracker_list;                                    \
   torrent::TrackerController tracker_controller(&tracker_list);         \
                                                                         \
@@ -23,23 +24,19 @@ CPPUNIT_TEST_SUITE_REGISTRATION(tracker_controller_test);
   int enabled_counter = 0;                                              \
   int disabled_counter = 0;                                             \
                                                                         \
-  tracker_controller.slot_success() = std::bind(&increment_value, &success_counter); \
-  tracker_controller.slot_failure() = std::bind(&increment_value, &failure_counter); \
-  tracker_controller.slot_timeout() = std::bind(&increment_value, &timeout_counter); \
-  tracker_controller.slot_tracker_enabled() = std::bind(&increment_value, &enabled_counter); \
-  tracker_controller.slot_tracker_disabled() = std::bind(&increment_value, &disabled_counter); \
+  tracker_controller.slot_success() = tr1::bind(&increment_value, &success_counter); \
+  tracker_controller.slot_failure() = tr1::bind(&increment_value, &failure_counter); \
+  tracker_controller.slot_timeout() = tr1::bind(&increment_value, &timeout_counter); \
+  tracker_controller.slot_tracker_enabled() = tr1::bind(&increment_value, &enabled_counter); \
+  tracker_controller.slot_tracker_disabled() = tr1::bind(&increment_value, &disabled_counter); \
                                                                         \
-  tracker_list.slot_success() = std::bind(&torrent::TrackerController::receive_success, &tracker_controller, std::placeholders::_1, std::placeholders::_2); \
-  tracker_list.slot_failure() = std::bind(&torrent::TrackerController::receive_failure, &tracker_controller, std::placeholders::_1, std::placeholders::_2); \
-  tracker_list.slot_tracker_enabled()  = std::bind(&torrent::TrackerController::receive_tracker_enabled, &tracker_controller, std::placeholders::_1); \
-  tracker_list.slot_tracker_disabled() = std::bind(&torrent::TrackerController::receive_tracker_disabled, &tracker_controller, std::placeholders::_1);
-
-#define TRACKER_INSERT(group, name)                             \
-  TrackerTest* name = new TrackerTest(&tracker_list, "");       \
-  tracker_list.insert(group, name);
+  tracker_list.slot_success() = tr1::bind(&torrent::TrackerController::receive_success, &tracker_controller, tr1::placeholders::_1, tr1::placeholders::_2); \
+  tracker_list.slot_failure() = tr1::bind(&torrent::TrackerController::receive_failure, &tracker_controller, tr1::placeholders::_1, tr1::placeholders::_2); \
+  tracker_list.slot_tracker_enabled()  = tr1::bind(&torrent::TrackerController::receive_tracker_enabled, &tracker_controller, tr1::placeholders::_1); \
+  tracker_list.slot_tracker_disabled() = tr1::bind(&torrent::TrackerController::receive_tracker_disabled, &tracker_controller, tr1::placeholders::_1);
 
 #define TEST_SINGLE_BEGIN()                                             \
-  TRACKER_SETUP();                                                      \
+  TRACKER_CONTROLLER_SETUP();                                                      \
   TRACKER_INSERT(0, tracker_0_0);                                       \
                                                                         \
   tracker_controller.enable();                                          \
@@ -65,7 +62,7 @@ CPPUNIT_TEST_SUITE_REGISTRATION(tracker_controller_test);
   //CPPUNIT_ASSERT(tracker_controller.seconds_to_promicious_mode() != 0);
 
 #define TEST_MULTI3_BEGIN()                                             \
-  TRACKER_SETUP();                                                      \
+  TRACKER_CONTROLLER_SETUP();                                                      \
   TRACKER_INSERT(0, tracker_0_0);                                       \
   TRACKER_INSERT(0, tracker_0_1);                                       \
   TRACKER_INSERT(1, tracker_1_0);                                       \
@@ -111,11 +108,14 @@ CPPUNIT_TEST_SUITE_REGISTRATION(tracker_controller_test);
 
 void
 tracker_controller_test::setUp() {
+  CPPUNIT_ASSERT(torrent::taskScheduler.empty());
+
   torrent::cachedTime = rak::timer::current();
 }
 
 void
 tracker_controller_test::tearDown() {
+  torrent::taskScheduler.clear();
 }
 
 void
@@ -157,7 +157,7 @@ tracker_controller_test::test_requesting() {
 
 void
 tracker_controller_test::test_timeout() {
-  TRACKER_SETUP();
+  TRACKER_CONTROLLER_SETUP();
   TRACKER_INSERT(0, tracker_0_0);
 
   CPPUNIT_ASSERT(enabled_counter == 1);
@@ -324,7 +324,7 @@ tracker_controller_test::test_send_task_timeout() {
 
 void
 tracker_controller_test::test_send_close_on_enable() {
-  TRACKER_SETUP();
+  TRACKER_CONTROLLER_SETUP();
   TRACKER_INSERT(0, tracker_0);
   TRACKER_INSERT(0, tracker_1);
   TRACKER_INSERT(0, tracker_2);
@@ -481,10 +481,23 @@ tracker_controller_test::test_multiple_send_stop() {
   tracker_controller.send_stop_event();
   tracker_controller.disable();
   CPPUNIT_ASSERT((tracker_controller.flags() & torrent::TrackerController::mask_send) == torrent::TrackerController::flag_send_stop);
+  TEST_MULTI3_IS_BUSY("01011", "10011");
 
-  tracker_controller.enable();
+  tracker_controller.enable(torrent::TrackerController::enable_dont_reset_stats);
+  TEST_MULTI3_IS_BUSY("00000", "00000");
   CPPUNIT_ASSERT((tracker_controller.flags() & torrent::TrackerController::mask_send) == 0);
 
+  tracker_controller.send_stop_event();
+  TEST_MULTI3_IS_BUSY("01011", "10011");
+
+  tracker_controller.disable();
+  tracker_controller.enable();
+  TEST_MULTI3_IS_BUSY("00000", "00000");
+  CPPUNIT_ASSERT((tracker_controller.flags() & torrent::TrackerController::mask_send) == 0);
+
+  tracker_controller.send_stop_event();
+  TEST_MULTI3_IS_BUSY("00000", "00000");
+
   // Test also that after closing the tracker controller, and
   // reopening it a 'send stop' event causes no tracker to be busy.
 
@@ -684,7 +697,7 @@ tracker_controller_test::test_disable_tracker() {
 
 void
 tracker_controller_test::test_new_peers() {
-  TRACKER_SETUP();
+  TRACKER_CONTROLLER_SETUP();
   TRACKER_INSERT(0, tracker_0);
 
   tracker_list.send_state_idx(0, torrent::Tracker::EVENT_NONE);
